package au.csiro.cmis.sact.xmlblaster;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.text.MessageFormat;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;

/**
 * <p>Title: Spread Benchmarks</p>
 * <p>Description: Test spread (http://www.spread.org) performance</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CSIRO</p>
 * @author Doug Palmer &lt;Doug.Palmer@csiro.au&gt;
 * @version 1.0
 */

public class Benchmark {
  /**
   * The name of the quality of service property "qos"
   */
  public static final String propQos = "qos";
  /**
   * The name of the sender name template
   */
  public static final String propSenderName = "senderName";
  /**
   * The name of the receiver name template
   */
  public static final String propReceiverName = "receiverName";
  /**
   * The name of the group name template
   */
  public static final String propGroupName = "groupName";
  /**
   * The name of the message size list
   */
  public static final String propMsgSize = "messageSize";
  /**
   * The name of the messages per second list
   */
  public static final String propMsgPerSecond = "messagesPerSecond";
  /**
   * The name of the number of senders list
   */
  public static final String propSenders = "senders";
  /**
   * The name of the number of receivers list
   */
  public static final String propReceivers = "receivers";
  /**
   * The name of the number of groups list
   */
  public static final String propGroups = "groups";
  /**
   * The name of the test length
   */
  public static final String propLength = "length";


  // The sender name template
  private MessageFormat senderName;
  // The receiver name template
  private MessageFormat receiverName;
  // The group name template
  private MessageFormat groupName;
  // The quality of service
  private String qos;
  // The number of messages to send per second
  private List msgsPerSecond;
  // The size of each message
  private List msgSize;
  // The number of senders
  private List numSenders;
  // The number of receivers
  private List numReceivers;
  // The number of groups
  private List numGroups;
  // The length of each test (seconds)
  private int testLength;
  // The current list of senders
  private List senders;
  // The current list of receivers
  private List receivers;

  // Decode a list of integers
  private List decodeIntList(String spec) {
    List list = new LinkedList();
    StringTokenizer tokenizer = new StringTokenizer(spec, ",", false);
    String v;

    while (tokenizer.hasMoreTokens()) {
      v = tokenizer.nextToken();
      list.add(new Integer(v));
    }
    return list;
  }

  // Decode a list of doubles
  private List decodeDoubleList(String spec) {
    List list = new LinkedList();
    StringTokenizer tokenizer = new StringTokenizer(spec, ",", false);
    String v;

    while (tokenizer.hasMoreTokens()) {
      v = tokenizer.nextToken();
      list.add(new Double(v));
    }
    return list;
  }

  /**
   * Constructor
   *
   * @param properties The file listing the properties for this test
   */
  public Benchmark(File properties) throws Exception {
    Properties props;

    props = new Properties();
    props.load(new FileInputStream(properties));
    this.qos = props.getProperty(this.propQos, "volatile");
    this.senderName = new MessageFormat(props.getProperty(this.propSenderName, "Sender{0}"));
    this.receiverName = new MessageFormat(props.getProperty(this.propReceiverName, "Receiver{0}"));
    this.groupName = new MessageFormat(props.getProperty(this.propGroupName, "BenchmarkGroup{0}"));
    this.msgSize = this.decodeIntList(props.getProperty(this.propMsgSize, "4096"));
    this.msgsPerSecond = this.decodeDoubleList(props.getProperty(this.propMsgPerSecond, "10,50,100,500"));
    this.numSenders = this.decodeIntList(props.getProperty(this.propSenders, "1"));
    this.numReceivers = this.decodeIntList(props.getProperty(this.propReceivers, "1"));
    this.numGroups = this.decodeIntList(props.getProperty(this.propGroups, "1"));
    this.testLength = Integer.parseInt(props.getProperty(this.propLength, "120"));
  }

  // Set up a test
  private void initTest(int msize, double mps, int nsenders, int nreceivers, int ngroups) {
    int i, group = 0;
    String n, g;

    this.senders = new LinkedList();
    this.receivers = new LinkedList();

    for (i = 0; i < nsenders; i++) {
      n = this.senderName.format(new Object[] {new Integer(i)});
      g = this.groupName.format(new Object[] {new Integer(group)});
      this.senders.add(new Sender(n, g, this.qos, mps / nsenders, msize));
      group = (group + 1) % ngroups;
    }
    for (i = 0; i < nreceivers; i++) {
      n = this.receiverName.format(new Object[] {new Integer(i)});
      g = this.groupName.format(new Object[] {new Integer(group)});
      this.receivers.add(new Receiver(n, g, this.qos));
      group = (group + 1) % ngroups;
    }
  }

  // Run the test and print the results
  private void runTest(int msize, double mps, int nsenders, int nreceivers, int ngroups, PrintWriter rs) throws Exception {
    Iterator i;
    Sender sender;
    Receiver receiver;
    long scount, rcount;
    long startTime, stopTime;

    this.initTest(msize, mps, nsenders, nreceivers, ngroups);

    // Start the stuff running
    i = this.senders.iterator();
    while (i.hasNext()) {
      sender = (Sender) i.next();
      sender.connect();
    }
    i = this.receivers.iterator();
    while (i.hasNext()) {
      receiver = (Receiver) i.next();
      receiver.connect();
    }
    i = this.receivers.iterator();
    while (i.hasNext()) {
      receiver = (Receiver) i.next();
      receiver.start();
    }
    startTime = System.currentTimeMillis();
    i = this.senders.iterator();
    while (i.hasNext()) {
      sender = (Sender) i.next();
      sender.start();
    }

    // Wait for the length of the test
    try {
      Thread.currentThread().sleep(this.testLength * 1000);
    } catch (InterruptedException ex) {
    }

    // Stop everything
    i = this.senders.iterator();
    while (i.hasNext()) {
      sender = (Sender) i.next();
      sender.stop();
    }
    while (i.hasNext()) {
      receiver = (Receiver) i.next();
      receiver.stop();
    }
    stopTime = System.currentTimeMillis();
    scount = 0;
    rcount = 0;
    i = this.senders.iterator();
    while (i.hasNext()) {
      sender = (Sender) i.next();
      sender.disconnect();
      scount += sender.getCount();
    }
    i = this.receivers.iterator();
    while (i.hasNext()) {
      receiver = (Receiver) i.next();
      receiver.disconnect();
      rcount += receiver.getCount();
    }

    rs.print(msize);
    rs.print(",");
    rs.print(mps);
    rs.print(",");
    rs.print(nsenders);
    rs.print(",");
    rs.print(nreceivers);
    rs.print(",");
    rs.print(ngroups);
    rs.print(",");
    rs.print(stopTime - startTime);
    rs.print(",");
    rs.print(scount);
    rs.print(",");
    rs.print(rcount);
    rs.println();
  }

  // Run all the tests
  public void runTests(PrintWriter rs) throws Exception {
    Iterator isize,imps,isenders,ireceivers,igroups;
    int msize,nsenders,nreceivers,ngroups;
    double mps;

    rs.println("Message Size,Messages Per Second,Senders,Receivers,Groups,Test Time,Sent,Recieved");
    isize = this.msgSize.iterator();
    while (isize.hasNext()) {
      msize = ((Integer) isize.next()).intValue();
      imps = this.msgsPerSecond.iterator();
      while (imps.hasNext()) {
        mps = ((Double) imps.next()).doubleValue();
        isenders = this.numSenders.iterator();
        while (isenders.hasNext()) {
          nsenders = ((Integer) isenders.next()).intValue();
          ireceivers = this.numReceivers.iterator();
          while (ireceivers.hasNext()) {
            nreceivers = ((Integer) ireceivers.next()).intValue();
            igroups = this.numGroups.iterator();
            while (igroups.hasNext()) {
              ngroups = ((Integer) igroups.next()).intValue();
              this.runTest(msize, mps, nsenders, nreceivers, ngroups, rs);
            }
          }
        }
      }
    }
  }

  public static void main(String[] args) {
    Benchmark benchmark;
    File props;
    File output;
    PrintWriter rs;

    if (args.length != 2) {
      System.err.println("Usage: au.csiro.cmis.sact.spread.Benchmark props output");
      System.exit(1);
    }
    try {
      props = new File(args[0]);
      output = new File(args[1]);
      rs = new PrintWriter(new FileOutputStream(output), true);
      benchmark = new Benchmark(props);
      benchmark.runTests(rs);
    } catch (Exception ex) {
      System.err.println(ex.getMessage());
      ex.printStackTrace(System.err);
    }
  }
}