package au.csiro.cmis.sact.xmlblaster;

import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;
import org.xmlBlaster.util.XmlBlasterException;
import org.xmlBlaster.engine.helper.MessageUnit;
import org.xmlBlaster.client.PublishKeyWrapper;
import org.xmlBlaster.client.PublishQosWrapper;
import org.xmlBlaster.client.protocol.XmlBlasterConnection;

/**
 * <p>Title: Spread Benchmarks</p>
 * <p>Description: Test spread (http://www.spread.org) performance</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CSIRO</p>
 * @author Doug Palmer &lt;Doug.Palmer@csiro.au&gt;
 * @version 1.0
 *
 * <p>
 * A sender of spread messages.
 * The sender can be parameterised to
 */

public class Sender {

  // A task that sends a message
  private class SenderTask extends TimerTask {
    // Run this process
    public void run() {
      try {
        Sender.this.connection.publish(Sender.this.messageFactory.getClone());
        Sender.this.count++;
      } catch (Exception ex) {
        System.err.println(ex.getMessage());
        ex.printStackTrace(System.err);
        this.cancel();
      }
    }
  }

  // The name of the sender
  private String name;
  // The group to join
  private String groupName;
  // The quality of service
  private String qos;
  // The number of messages to send per second
  private double msgsPerSecond;
  // The size of each message
  private int msgSize;
  // The xmlBlaster connection
  private XmlBlasterConnection connection;
  // The xmlBlaster publish key
  private PublishKeyWrapper publishKey;
  // The xmlBlaster publish qos
  private PublishQosWrapper publishQos;
  // The standard message to use
  private MessageUnit messageFactory;
  // The timer task
  private Timer timer;
  // The message count
  private int count;

  // Initialise the message factory
  private void initMessageFactory() {
    byte[] data = new byte[Sender.this.msgSize];
    Random rand = new Random();

    rand.nextBytes(data);
    this.messageFactory = new MessageUnit(this.publishKey, data, this.publishQos);
  }

  /**
   * Constructor
   *
   * @param name The name of the sender
   * @param groupName The group name to send to
   * @param server The hostname of the daemon to connect to
   * @param port The port number to connect to
   * @param mps Number of messages per second
   * @param msize The size (in bytes) of each message
   */
  public Sender(String name, String groupName, String qos, double mps, int msize) {
    this.name = name;
    this.groupName = groupName;
    this.qos = qos;
    this.msgsPerSecond = mps;
    this.msgSize = msize;
    this.count = 0;
  }

  /**
   * Connect this sender to the spread daemon
   */
  public void connect() throws XmlBlasterException {
    if (this.connection == null) {
      this.connection = new XmlBlasterConnection();
      this.connection.connect(null, null);
      this.publishKey = new PublishKeyWrapper(this.groupName, "application/octet-stream");
      this.publishQos = new PublishQosWrapper();
      this.publishQos.setDurable(this.qos.equalsIgnoreCase("durable"));
    }
  }

  /**
   * Disconnect from the spread daemon
   */
  public void disconnect() throws XmlBlasterException {
    if (this.connection != null) {
      this.connection.disconnect(null);
      this.connection = null;
    }
  }

  /**
   * Start sending messages
   */
  public void start() throws XmlBlasterException {
    long period = Math.round(1000.0 / this.msgsPerSecond);

    this.initMessageFactory();
    if (period <= 0)
      period = 1;
    if (this.timer == null) {
      this.timer = new Timer();
      this.timer.scheduleAtFixedRate(new SenderTask(), period / 2, period);
    }
  }

  /**
   * Stop sending messages
   */
  public void stop() throws XmlBlasterException  {
    if (this.timer != null) {
      this.timer.cancel();
      this.timer = null;
    }
  }

  /**
   * Get the count of messages
   *
   * @return The number of messages sent
   */
  public int getCount() {
    return this.count;
  }
}